use mox::value::Value;
use mox::{Chunk, ConversionError, OptCode};

pub fn disassemble_chunk(chunk: &Chunk, name: &str) {
    // println!("{:?}", chunk);
    println!("== {} ==", name);
    let mut offset = 0;
    while offset < chunk.code.len() {
        offset = disassemble_instruction(&chunk, offset);
    }
}
fn disassemble_instruction(chunk: &Chunk, offset: usize) -> usize {
    print!("{:04} ", offset);
    let instruction: Result<OptCode, ConversionError> = chunk.code[offset].try_into();
    match instruction {
        Ok(code) => match code {
            OptCode::OpConstant => {
                return constant_instruction("OpConstant", chunk, offset);
            }
            OptCode::OpReturn => {
                return simple_instruction("OpReturn", offset);
            }
        },
        Err(e) => {
            println!("Unknown optcode {:?}", e);
            return offset + 1;
        }
    }
}

fn simple_instruction(name: &str, offset: usize) -> usize {
    println!("{}", name);
    return offset + 1;
}
fn constant_instruction(name: &str, chunk: &Chunk, offset: usize) -> usize {
    let constant: u8 = chunk.code[offset + 1];
    print!("{:<16} {:>4} ", name, constant);
    print_value(chunk.constants[constant as usize]);
    println!();
    return offset + 2;
}
fn print_value(value: Value) {
    print!("{:}", value);
}
